;
title	general purpose utility routines
page	60,132
;*
;*	The routines in this module are designed to be general purpose
;*	in nature.  All are far routines, and can be used by application
;*	programs as desired via a far call.  Refer to the desired
;*	routine below for a description of the registers used.
;*
;*	***********************************************************
;*
;*	       Note:  This revision used for BIOS versions dated
;*		     9/19/83 and later.  Earlier versions had these
;*		     routines at FE00:1900h.
;*
;*	***********************************************************
;*
;*	date of this revision:	10/26/83
;*
;*	public and external symbol definitions
;*
public	qinit,qadd,qrem,play_tone,play_tune,print_message
public	msg_tbl
;
data	segment at 040h
extrn	equip_flag:word
data	ends
;
cseg	segment	common
assume	cs:cseg
;
	org	0DB0h		; start module at FE00:0D80
;
;**********************************************************************
;*
;*      The following routines are general purpose queue data
;*      handling routines.  They are designed to work on 16-bit
;*      word data variables, but can be easily changed to handle
;*      queues with varying data element sizes by adding another
;*      parameter for data element size, and adding a loop in the
;*      add and remove routines to transfer the specified number
;*      of bytes.
;*
;*      These routines were adapted from "SOFTWARE DESIGN AND
;*      EVALUATION:  AN INTRODUCTION TO SOFTWARE ENGINEERING" by
;*      Howard A. Sholl of the University of Connecticut.
;*
;*      Description of the queue data structure:
;*
;*      The queue data structure is implemented as a circular buffer.
;*      This structure is the well known FIFO where the first data
;*      element in is the first one out.  When the queue pointers get
;*      to the physical end of the buffer area, they are "wrapped
;*      around" and set back to the physical starting address of the
;*      buffer area.  When the add pointer and the remove pointer are
;*      at the same physical address, the queue can be either full or
;*      empty, depending on which pointer was advanced last.  A status
;*      variable is used to distinguish the correct condition of the
;*      queue.  Interrupts are disabled at critical points of the
;*      algorithms to prevent an interrupting routine from accessing
;*      the same queue in the middle of an insertion or deletion of a
;*      data element.
;*
;*      Data elements are added to the bottom of the queue, and are
;*      removed from the top of the queue.
;*
;*      Queues are referenced via a queue parameter block pointed
;*      to by the DS and BX registers.  The format of the queue
;*      parameter block is as follows:
;*
;*              word 0 - segment of base address of queue
;*              word 1 - offset of base address of queue
;*              word 2 - offset of pointer to top of queue
;*              word 3 - offset of pointer to bottom of queue
;*              word 4 - length of queue
;*              word 5 - queue status (0=empty,-1=full)
;*
;**********************************************************************
;*
;*      Equates used by the queue routines:
;*
qseg            equ     0       ; segment of queue buffer area
qoff            equ     2       ; offset of base of queue buffer area
qtop            equ     4       ; offset of pointer to top of queue
qbot            equ     6       ; offset of pointer to bottom of queue
qlen            equ     8       ; number of bytes allocated to queue
qstat           equ     10      ; status flags for the queue
qempty          equ     0       ; the queue is empty
qfull           equ     -1      ; the queue is full
;
;**********************************************************************
;*
;*      Subroutine name - qinit
;*
;*      Purpose - to initialize a queue parameter block for an empty
;*                queue.
;*
;*      On entry:
;*
;*              BX contains the offset to the queue buffer,
;*              SI contains the offset to the queue parameter block,
;*              DS contains the segment value for the parameter block,
;*              ES contains the segment value for the queue buffer,
;*              AX contains the length of the buffer.
;*
;*      On exit:
;*
;*              The queue parameter block will be initialized for
;*              an empty queue.  Nothing is returned.
;*
;**********************************************************************
;
qinit	proc	far
;
        pushf                   ; save flags and disable system
        cli                     ; interrupts until done
        mov     qlen[si],ax     ; store length of buffer
        mov     qoff[si],bx     ; offset of base of buffer
        mov     ax,es           ; segment of buffer area
        mov     qseg[si],ax     ; segment value for queue buffer
        mov     qtop[si],bx     ; set top pointer to base
        mov     qbot[si],bx     ; set bottom pointer to base
        mov     word ptr qstat[si],qempty       ; set status to empty
        popf                    ; restore flags
        ret
;
qinit	endp
;
;
;**********************************************************************
;*
;*      Subroutine name - qadd
;*
;*      Purpose - to add a data element to a queue data structure which
;*                is defined by a queue parameter block pointed to by
;*                the DS:SI registers.
;*
;*      On entry:
;*
;*              AX contains the data word,
;*              SI contains the offset of the queue parameter block,
;*              DS contains the segment of the queue parameter block.
;*
;*      On exit:
;*
;*              The carry flag will be clear if add was successful,
;*              the carry flag will be set if queue was already full,
;*              the data element will be in the queue if successfull.
;*
;**********************************************************************
;
qadd	proc	far
;
        push    es
        push    bx
        pushf           ; save system flags
        mov     bx,qseg[si]     ; set up the segment register
        mov     es,bx
        cmp     word ptr qstat[si],qfull ; error if queue is full
        jne     qaddok
;
        popf                    ; restore system flags
        stc                     ; set carry flag for error
	jmp	short qadd_exit
;
qaddok:
;
        cli             ; turn off interrupts during the add operation
        mov     word ptr qstat[si],1    ; indicate non-full/non-empty
        mov     bx,qbot[si]     ; get pointer to bottom of queue
        mov     es:[bx],ax      ; add the data word to the queue
        add     bx,2            ; and increment pointer
        mov     qbot[si],bx
;*
;*      check for queue wrap-around, and reset bottom pointer
;*      back to the base of the queue if so.
;*
        mov     ax,qoff[si]     ; base address
        add     ax,qlen[si]     ; add the length of the queue
        cmp     ax,bx           ; compare to current offset value
        jne     qa1
;
        mov     ax,qoff[si]     ; reset bottom ptr to base offset
        mov     qbot[si],ax
;*
;*      check if queue full condition has occurred and set status
;*      flag if so.
;*
qa1:    mov     ax,qtop[si]
        cmp     ax,qbot[si]      ; queue is full if top=bottom
        jne     qa2
        mov     word ptr qstat[si],qfull ; set status to qfull condition
;
qa2:    popf                    ; restore system flags
        clc                     ; clear carry flag to indicate a
;
qadd_exit:
;
        pop     bx              ; sucessfull add operation
        pop     es
        ret
;
qadd	endp
;
;**********************************************************************
;*
;*      Subroutine name - qrem
;*
;*      Purpose - to remove the next element from the queue data
;*                structure defined to by the queue parameter block
;*                pointed to by the DS:SI registers.
;*
;*      On entry:
;*
;*              SI contains the offset of the queue parameter block,
;*              DS contains the segment of the parameter block.
;*
;*      On exit:
;*
;*              The carry flag will be set if the queue was empty,
;*              the carry flag will be clear if data is available, and
;*              the data word will be returned in the AX register.
;*
;**********************************************************************
;
qrem	proc	far
;
        push    es
        push    bx
        pushf           ; save system flags
        mov     ax,qseg[si]     ; set up the segment of queue
        mov     es,ax
        cmp     word ptr qstat[si],qempty ; error if queue is empty
        jne     qremok
;
        popf                    ; restore system flags
        stc                     ; set carry flag for error
	jmp	short qrem_exit
;
qremok:
;
        cli     ; turn off interrupts during this operation
        mov     word ptr qstat[si],1    ; indicate not full or empty
        mov     bx,qtop[si]     ; get pointer to top of queue
        mov     ax,es:[bx]              ; get the next data word
        add     bx,2            ; and increment the pointer
        mov     qtop[si],bx
;*
;*      check for wrap-around of queue
;*
        push    ax              ; save the data word
        mov     ax,qoff[si]     ; get offset of base pointer
        add     ax,qlen[si]     ; add length of queue
        cmp     ax,bx           ; compare to offset of top ptr
        jne     qr1
;
        mov     ax,qoff[si]     ; reset the top pointer
        mov     qtop[si],ax
;*
;*      check for a queue empty condition and set status if so
;*
qr1:    mov     ax,qtop[si]     ; top pointer
        cmp     ax,qbot[si]     ; equal to bottom ptr?
        jne     qr2
        mov     word ptr qstat[si],qempty ; clear status flag
;
qr2:    pop     ax              ; restore data word
        popf                    ; restore system flags
        clc                     ; indicate sucessful operation
;
qrem_exit:
;
        pop     bx
        pop     es
        ret
;
qrem	endp
;*
;********************************************************************
;*
;*	Subroutine name - print_message
;*
;*	Purpose - to print an ascii message at the console device
;*
;*	On entry - AL contains the message number
;*		   DS:SI points to the message table
;*
;*	On exit - the message will be displayed, no registers destroyed
;*
;**********************************************************************
;*
print_message	proc	far
;
assume	ds:data
;
	push	ax
	push	bx
	push	si
	shl	al,1			; use word offsets
	xor	ah,ah			; clear just in case
	mov	bx,ax			; get base of table
	mov	ah,15			; get page number
	int	010h
;
set_ptr:
;
	mov	si,[bx+si]		; set up message pointer
;
msg_loop:
;
	mov	al,[si]			; get next character
	inc	si			; point to next character
	cmp	al,0
	je	pmsg_exit		; done if a 0
;
	mov	ah,014			; use tty output mode
	int	010h
	jmp	short msg_loop
;
pmsg_exit:
;
	pop	si
	pop	bx
	pop	ax
	ret
;
print_message	endp
;
;*	Rom-based messages go here
;*
msg_tbl		dw	msg0
		dw	msg1
		dw	msg2
		dw	msg3
		dw	msg4
		dw	msg5
		dw	msg6
		dw	msg7
		dw	msg8
		dw	msg9
msg_tbl_len	equ	offset $ - offset msg_tbl
;
cr	equ	0dh
lf	equ	0ah
;
msg0	db	cr,lf,'MPX-16 Loader  10/24/83',cr,lf,cr,lf,0
msg1	db	cr,lf,'HW Failure',cr,lf,0
msg2	db	cr,lf,'Boot Failure',cr,lf,0
msg3	db	cr,lf,'Parity Error',cr,lf,0
msg4	db	cr,lf,'Interrupt Trap',cr,lf,0
msg5	db	cr,lf,'user msg1 ',cr,lf,0
msg6	db	cr,lf,'user msg2 ',cr,lf,0
msg7	db	cr,lf,'user msg3 ',cr,lf,0
msg8	db	cr,lf,'user msg4 ',cr,lf,0
msg9	db	cr,lf,'J. Norris ',cr,lf,0
;*
;******************************************************************
;*
;*	Subroutine name - play_tone
;*
;*	Purpose - to output a tone of a specified frequency and time
;*
;*	On entry:
;*
;*		DH determines mode of operation:
;*			3 = use duration in BX,
;*			1 = turn on and leave on
;*			2 = turn off
;*		DL contains the speaker channel (0,1,2)
;*		AX contains the frequency in hertz,
;*		BX contains the duration in units of 0.010 seconds
;*
;*	On exit:
;*
;*		The tone will have been sounded, AX, BX destroyed
;*
;**********************************************************************
;*
play_tone	proc	far
;
	push	cx
	push	si
	push	di
	push	dx		; save mode control byte
	xor	dh,dh		; strip off mode information
	shl	dl,1		; *2 for word offsets
	mov	di,dx		; save speaker channel
	mov	si,ax		; save frequency
	mov	al,cs:spk_mode[di]	; get mode control byte
	mov	dx,0243h	; 8253 control port
	out	dx,al
;
;*	count = 1193182/freq
;*
	mov	ax,034deh	; 1,193,182
	mov	dx,012h
	div	si		; freq in hertz is in si
;
	pop	dx
	push	dx
	and	dx,0300h	; strip off extra bits
	mov	si,dx		; save in si
;
	mov	dx,cs:spk_port[di]	; get correct timer channel
	out	dx,al
	mov	al,ah
	out	dx,al
;
	mov	dx,0261h	; speaker control port
	in	al,dx
	test	si,0100h	; turn on bit set?
	je	off_test
;
	or	al,cs:spk_on[di]	; turn on the specified channel
	out	dx,al
	test	si,0200h	; use delay if off bit is also set
	je	spk_exit
;
;
wait_10:			; 10 millisecond wait loop
	mov	cx,2801
on_loop:
	loop	on_loop
;
	dec	bx		; repeat until time is done
	jne	wait_10
;
off_test:
	test	si,0200h	; off bit set?
	je	spk_exit
;
	and	al,cs:spk_off[di]	; turn off the specified channel
	out	dx,al
;
spk_exit:
	pop	dx
	pop	di
	pop	si
	pop	cx
	ret
;*
;*	Data tables for speaker routines follow
;*
spk_on		db	3,0,0ch,0,030h,0
spk_off		db	0fch,0,0f3h,0,0cfh,0
spk_port	dw	0242h,0240h,0241h
spk_mode	db	0b6h,0,036h,0,056h,0
;
play_tone	endp
;*
;********************************************************************
;*
;*	Subroutine name - play_tune
;*
;*	Purpose - to play a table based tune via the keyboard adapter
;*
;*	On entry:
;*
;*		SI points to a word table of frequencies (in hertz)
;*		DI points to a word table of time durations (in .010 s)
;*		DL indicates the speaker channel (0,1,2)
;*
;*	On exit:
;*
;*		The tune will have played, SI and DI destroyed.
;*
;*********************************************************************
;
play_tune	proc	far
;
	push	ax
	push	bx
	push	dx
;
get_freq:
	mov	ax,[si]		; si is a pointer to frequency table
	cmp	ax,0		; 0 word indicates end of tune
	je	end_of_tune
;
	mov	dh,3		; indicate use timed mode
	mov	bx,[di]		; di is a pointer to a time table
	call	play_tone
	add	si,2
	add	di,2		; bump pointers
	jmp	short get_freq
;
end_of_tune:
	pop	dx
	pop	bx
	pop	ax
	ret
;
play_tune	endp
;
	org	0FF5H
	db	'10/26/83'	; release date of this eprom
cseg	ends
	end
